//+------------------------------------------------------------------+
//|                                                      ProjectName |
//|                                      Copyright 2012, CompanyName |
//|                                       http://www.companyname.net |
//+------------------------------------------------------------------+
#property description "RSI+Stochastic"
#property version   "1.00"
#property strict

//--- Inputs
extern bool       useTimeFilter     = true;
extern int        HourBegin         = 4;
extern int        HourEnd           = 18;

input double Lots          =0.1;
input double MaximumRisk   =0.02;
input double DecreaseFactor=3;
extern double TrailingStop = 200;
input double TakeProfit    =500;
extern int   MagicNumber=777;
extern string stoset="sto set";
extern int   K_Period = 5;
extern int   D_Period = 3;
extern int   Slowing=3;
extern int   Sto_SellLevel= 80;
extern int   Sto_BuyLevel = 20;
extern string rsiset="rsi set";
extern int   RSI_Period=14;
extern int   RSI_SellLevel= 70;
extern int   RSI_BuyLevel = 30;
//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
   double lot=Lots;
   int    orders=HistoryTotal();     // history orders total
   int    losses=0;                  // number of losses orders without a break
   double TickVal=MarketInfo(Symbol(),MODE_TICKVALUE);
//--- select lot size
//lot=NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,1);
   if(MaximumRisk>0)
     {
      double _SL=TrailingStop;
      if(_SL==0)_SL=iATR(Symbol(),PERIOD_D1,8,1)/Point;
      lot=AccountFreeMargin()/100*MaximumRisk/TickVal/(_SL);
     }
//--- calcuulate number of losses orders without a break
   if(DecreaseFactor>0)
     {
      for(int i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false)
           {
            Print("Error in history!");
            break;
           }
         if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL)
            continue;
         //---
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++;
        }
      if(losses>1)
         lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,2);
     }
//--- return lot size
   if(lot>MarketInfo(Symbol(),MODE_MAXLOT)) lot=MarketInfo(Symbol(),MODE_MAXLOT);
   if(lot<MarketInfo(Symbol(),MODE_MINLOT)) lot=MarketInfo(Symbol(),MODE_MINLOT);
   return(lot);
  }
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
  {
   double RSI,StochasticPrevious,Stochastic,SignalPrevious,Signal;
   int    res;
//--- go trading only for first tiks of new bar
   if(Volume[0]>1 || !WorkTime()) return;
//--- to simplify the coding and speed up access data are put into internal variables
   RSI=iRSI(NULL,0,RSI_Period,PRICE_CLOSE,0);
   Stochastic=iStochastic(NULL,0,K_Period,D_Period,Slowing,MODE_SMA,STO_LOWHIGH,MODE_MAIN,0);
   StochasticPrevious=iStochastic(NULL,0,K_Period,D_Period,Slowing,MODE_SMA,STO_LOWHIGH,MODE_MAIN,1);
   Signal=iStochastic(NULL,0,K_Period,D_Period,Slowing,MODE_SMA,STO_LOWHIGH,MODE_SIGNAL,0);
   SignalPrevious=iStochastic(NULL,0,K_Period,D_Period,Slowing,MODE_SMA,STO_LOWHIGH,MODE_SIGNAL,1);
//--- sell conditions
   if(RSI>RSI_SellLevel && Stochastic>Sto_SellLevel && StochasticPrevious>SignalPrevious && Stochastic<Signal)
     {
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,0,Bid-TakeProfit*Point,"Genie",MagicNumber,0,Red);
      return;
     }
//--- buy conditions
   if(RSI<RSI_BuyLevel && Stochastic<Sto_BuyLevel && StochasticPrevious<SignalPrevious && Stochastic>Signal)
     {
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,0,Ask+TakeProfit*Point,"Genie",MagicNumber,0,Blue);
      return;
     }
//---
  }
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
//--- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//---
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderMagicNumber()!=MagicNumber || OrderSymbol()!=Symbol()) continue;
      //--- check order type 
      if(OrderType()==OP_BUY)
        {
         if(OrderStopLoss()<Bid-Point*TrailingStop)
           {
            //--- modify order and exit
            if(!OrderModify(OrderTicket(),OrderOpenPrice(),Bid-Point*TrailingStop,OrderTakeProfit(),0,Green))
               Print("OrderModify error ",GetLastError());
            return;
           }
         //if(Open[1]>Close[1] && Bid-OrderOpenPrice()>Ask-Bid)
         //  {
         //   if(!OrderClose(OrderTicket(),OrderLots(),Bid,3,White))
         //      Print("OrderClose error ",GetLastError());
         //  }
         break;
        }
      if(OrderType()==OP_SELL)
        {
         if((OrderStopLoss()>(Ask+Point*TrailingStop)) || (OrderStopLoss()==0))
           {
            //--- modify order and exit
            if(!OrderModify(OrderTicket(),OrderOpenPrice(),Ask+Point*TrailingStop,OrderTakeProfit(),0,Red))
               Print("OrderModify error ",GetLastError());
            return;
           }
         //if(Open[1]<Close[1] && OrderOpenPrice()-Ask>Ask-Bid)
         //  {
         //   if(!OrderClose(OrderTicket(),OrderLots(),Ask,3,White))
         //      Print("OrderClose error ",GetLastError());
         //  }
         break;
        }
     }
//---
  }
//+------------------------------------------------------------------+
//| OnTick function                                                  |
//+------------------------------------------------------------------+
void OnTick()
  {
//--- check for history and trading
   if(Bars<100 || IsTradeAllowed()==false)
      return;
//--- calculate open orders by current symbol
   if(CalculateCurrentOrders(Symbol())==0) CheckForOpen();
   else                                    CheckForClose();
//---
  }
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
  {
   int buys=0,sells=0;
//---
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber)
        {
         if(OrderType()==OP_BUY)  buys++;
         if(OrderType()==OP_SELL) sells++;
        }
     }
//--- return orders volume
   if(buys>0) return(buys);
   else       return(-sells);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool WorkTime()
  {
   if(!useTimeFilter) return true;
   int currHour=TimeHour(TimeCurrent());
   if(HourBegin<HourEnd && currHour>=HourBegin && currHour<HourEnd)return true;
   if(HourBegin>=HourEnd && (currHour>=HourBegin || currHour<HourEnd))return true;
   return false;
  }
//+------------------------------------------------------------------+
